@@ -18,13 +18,14 @@ const data_size_t DCGCalculator::kMaxPosition = 10000;
 
 
 void DCGCalculator::DefaultEvalAt(std::vector<int>* eval_at) {
-  if (eval_at->empty()) {
+  auto& ref_eval_at = *eval_at;
+  if (ref_eval_at.empty()) {
     for (int i = 1; i <= 5; ++i) {
-      eval_at->push_back(i);
+      ref_eval_at.push_back(i);
     }
   } else {
     for (size_t i = 0; i < eval_at->size(); ++i) {
-      CHECK(eval_at->at(i) > 0);
+      CHECK_GT(ref_eval_at[i], 0);
     }
   }
 }
@@ -105,27 +106,6 @@ void DCGCalculator::CalMaxDCG(const std::vector<data_size_t>& ks,
   }
 }
 
-
-double DCGCalculator::CalDCGAtK(data_size_t k, const label_t* label,
-                                const double* score, data_size_t num_data) {
-  // get sorted indices by score
-  std::vector<data_size_t> sorted_idx(num_data);
-  for (data_size_t i = 0; i < num_data; ++i) {
-    sorted_idx[i] = i;
-  }
-  std::stable_sort(sorted_idx.begin(), sorted_idx.end(),
-                   [score](data_size_t a, data_size_t b) {return score[a] > score[b]; });
-
-  if (k > num_data) { k = num_data; }
-  double dcg = 0.0f;
-  // calculate dcg
-  for (data_size_t i = 0; i < k; ++i) {
-    data_size_t idx = sorted_idx[i];
-    dcg += label_gain_[static_cast<int>(label[idx])] * discount_[i];
-  }
-  return dcg;
-}
-
 void DCGCalculator::CalDCG(const std::vector<data_size_t>& ks, const label_t* label,
                            const double * score, data_size_t num_data, std::vector<double>* out) {
   // get sorted indices by score
@@ -151,6 +131,19 @@ void DCGCalculator::CalDCG(const std::vector<data_size_t>& ks, const label_t* la
   }
 }
 
+void DCGCalculator::CheckMetadata(const Metadata& metadata, data_size_t num_queries) {
+  const data_size_t* query_boundaries = metadata.query_boundaries();
+  if (num_queries > 0 && query_boundaries != nullptr) {
+    for (data_size_t i = 0; i < num_queries; i++) {
+      data_size_t num_rows = query_boundaries[i + 1] - query_boundaries[i];
+      if (num_rows > kMaxPosition) {
+        Log::Fatal("Number of rows %i exceeds upper limit of %i for a query", static_cast<int>(num_rows), static_cast<int>(kMaxPosition));
+      }
+    }
+  }
+}
+
+
 void DCGCalculator::CheckLabel(const label_t* label, data_size_t num_data) {
   for (data_size_t i = 0; i < num_data; ++i) {
     label_t delta = std::fabs(label[i] - static_cast<int>(label[i]));
@@ -158,8 +151,13 @@ void DCGCalculator::CheckLabel(const label_t* label, data_size_t num_data) {
       Log::Fatal("label should be int type (met %f) for ranking task,\n"
                  "for the gain of label, please set the label_gain parameter", label[i]);
     }
-    if (static_cast<size_t>(label[i]) >= label_gain_.size() || label[i] < 0) {
-      Log::Fatal("label (%d) excel the max range %d", label[i], label_gain_.size());
+
+    if (label[i] < 0) {
+      Log::Fatal("Label should be non-negative (met %f) for ranking task", label[i]);
+    }
+
+    if (static_cast<size_t>(label[i]) >= label_gain_.size()) {
+      Log::Fatal("Label %zu is not less than the number of label mappings (%zu)", static_cast<size_t>(label[i]), label_gain_.size());
     }
   }
 }
