@@ -67,7 +67,7 @@ class DatasetLoader {
   /*! \brief Random generator*/
   Random random_;
   /*! \brief prediction function for initial model */
-  const PredictFunction& predict_fun_;
+  const PredictFunction predict_fun_;
   /*! \brief number of classes */
   int num_class_;
   /*! \brief index of label column */
@@ -82,6 +82,8 @@ class DatasetLoader {
   std::vector<std::string> feature_names_;
   /*! \brief Mapper from real feature index to used index*/
   std::unordered_set<int> categorical_features_;
+  /*! \brief Whether to store raw feature values */
+  bool store_raw_;
 };
 
 }  // namespace LightGBM
