@@ -4,6 +4,7 @@ context("Learning to rank")
 TOLERANCE <- 1e-06
 
 ON_SOLARIS <- Sys.info()["sysname"] == "SunOS"
+ON_32_BIT_WINDOWS <- .Platform$OS.type == "windows" && .Machine$sizeof.pointer != 8L
 
 test_that("learning-to-rank with lgb.train() works as expected", {
     set.seed(708L)
@@ -48,14 +49,17 @@ test_that("learning-to-rank with lgb.train() works as expected", {
     }
     expect_identical(sapply(eval_results, function(x) {x$name}), eval_names)
     expect_equal(eval_results[[1L]][["value"]], 0.775)
-    if (!ON_SOLARIS) {
+    if (!(ON_SOLARIS || ON_32_BIT_WINDOWS)) {
         expect_true(abs(eval_results[[2L]][["value"]] - 0.745986) < TOLERANCE)
         expect_true(abs(eval_results[[3L]][["value"]] - 0.7351959) < TOLERANCE)
     }
 })
 
 test_that("learning-to-rank with lgb.cv() works as expected", {
-    testthat::skip_if(ON_SOLARIS, message = "Skipping on Solaris")
+    testthat::skip_if(
+        ON_SOLARIS || ON_32_BIT_WINDOWS
+        , message = "Skipping on Solaris and 32-bit Windows"
+    )
     set.seed(708L)
     data(agaricus.train, package = "lightgbm")
     # just keep a few features,to generate an model with imperfect fit
