@@ -1,6 +1,6 @@
 context(".METRICS_HIGHER_BETTER()")
 
-test_that(".METRICS_HIGHER_BETTTER() should be well formed", {
+test_that(".METRICS_HIGHER_BETTER() should be well formed", {
     metrics <- .METRICS_HIGHER_BETTER()
     metric_names <- names(.METRICS_HIGHER_BETTER())
     # should be a logical vector
